@@ -94,18 +94,25 @@ class IFourdigitsPortletTwitter(IPortletDataProvider):
                       it wont be shown, one per line"),
         required=False,)
 
-    count = schema.Int(
+    userdisplay = schema.Int(
         title=_(u'Number of items to display based on the username'),
         description=_(u'How many items to list based on the username.'),
         required=False,
         default=5,
     )
-    searchcount = schema.Int(
+    searchdisplay = schema.Int(
         title=_(u'Number of items to display based on the searchtext'),
         description=_(u'How many items to list based on the searchtext.'),
         required=False,
         default=5,
     )
+    searchlimit = schema.Int(
+        title=_(u'Number of items to search for, defaults to 40'),
+        description=_(u'Number of items to search for, defaults to 40'),
+        required=True,
+        default=40,
+    )
+
     language = schema.Text(
         title=_("Languagefilter"),
         description=_("Language ISO code for the tweets (e.g.: en, nl, fr), \
@@ -130,13 +137,15 @@ class Assignment(base.Assignment):
     implements(IFourdigitsPortletTwitter)
 
     def __init__(self, name=u"", username=u"", search=u"", filtertext="",
-                 count=5, searchcount=5, language="", userpictures=False):
+                 userdisplay=5, searchdisplay=5, searchlimit=40,
+                 language="", userpictures=False):
         self.name = name
         self.username = username
         self.search = search
         self.filtertext = filtertext
-        self.count = count
-        self.searchcount = searchcount
+        self.userdisplay = userdisplay
+        self.searchdisplay = searchdisplay
+        self.searchlimit = searchlimit
         self.language = language
         self.userpictures = userpictures
 
@@ -158,15 +167,15 @@ class Renderer(base.Renderer):
 
     render = ViewPageTemplateFile('fourdigitsportlettwitter.pt')
 
+    twapi = twitter.Api()
+
     @property
     def title(self):
         return self.data.name or _(u"Tweets")
 
     @property
     def available(self):
-        if len(self.getTweets()) > 0:
-            return True
-        return False
+        return True
 
     def expand(self, str):
         return expand_tweet(str)
@@ -175,92 +184,87 @@ class Renderer(base.Renderer):
         """Twitter messages"""
         return self.getTweets()
 
+    def getTweetsOfUser(self, username, userpictures):
+        """Return the tweets of a certain user"""
+        try:
+            tweets = self.twapi.GetUserTimeline(username)
+        except URLError:
+            tweets = []
+        return tweets
+
+    def getTweetsBySearch(self, searchterms, searchlimit, language):
+        """Return tweets based on a search query"""
+        searchterms = searchterms.encode('utf-8')
+        searchterms = searchterms.split('\n')
+        for searchterm in searchterms:
+            # get tweets per searchterm
+            try:
+                tweets = self.twapi.GetSearch(term=searchterm,
+                                              per_page=searchlimit,
+                                              lang=language)
+            except:
+                tweets = []
+        return tweets
+
     def getTweets(self):
         """get the tweets and filter them"""
         username = self.data.username
-        search = self.data.search
-        limit = self.data.count
-        searchlimit = self.data.searchcount
+        searchterms = self.data.search
+        userdisplay = self.data.userdisplay
+        searchdisplay = self.data.searchdisplay
+        searchlimit = self.data.searchlimit
         filtertext = self.data.filtertext
-        language = self.data.language
+        languages = self.data.language
         userpictures = self.data.userpictures
 
-        if filtertext:
-            filtertext = filtertext.lower()
-            filterlist = filtertext.split('\n')
         results = []
-        searchresults = []
-
-        twapi = twitter.Api()
         tweets = []
 
         # get tweets of username
         if username:
-            try:
-                tweets = twapi.GetUserTimeline(username)[:limit]
-                for tweet in tweets:
-                    picture = tweet.user.GetProfileImageUrl()
-                    tweet.text = "%s: %s" % (tweet.user.GetScreenName(),
-                                             tweet.text)
-                    if userpictures:
-                        tweet.picture = picture
-            except URLError:
-                tweets = []
+            tweets = self.getTweetsOfUser(username, userpictures)
+        tweets = tweets[:userdisplay]
 
+        searchresults = []
         # get tweets based on search
-        if search:
-            search = search.encode('utf-8')
-            search = search.split('\n')
-            if language:
-                if len(language) > 0:
-                    language = language.split('\n')
+        if searchterms:
+            if languages:
+                languages = languages.split('\n')
+                for lang in languages:
+                    lang = str(lang.encode('utf-8'))
+                    searchresults += self.getTweetsBySearch(searchterms,
+                                                            searchlimit,
+                                                            language=lang)
+            else:
+                searchresults += self.getTweetsBySearch(searchterms,
+                                                        searchlimit,
+                                                        language="")
+            # Only add tweets which are not already in the user results
+            filtered_results = [tweet for tweet
+                                      in searchresults
+                                      if not tweet in tweets]
+            tweets += filtered_results[:searchdisplay]
 
-            for searchterm in search:
-                # get tweets per language
-                if language:
-                    for lang in language:
-                        lang = str(lang.encode('utf-8'))
-
-                        try:
-                            searchresults = twapi.GetSearch(term=searchterm,
-                                                    per_page=searchlimit,
-                                                        lang=lang)
-                        except:
-                            searchresults = []
-
-                        for tweet in searchresults:
-                            username = tweet.user.GetScreenName()
-                            if picture:
-                                picture = tweet.user.GetProfileImageUrl()
-                            tweet.text = "%s: %s" % (username,
-                                                     tweet.text)
-                            tweet.picture = picture
-                    tweets += searchresults
-                else:
-                    try:
-                        searchresults = twapi.GetSearch(term=searchterm,
-                                                    per_page=searchlimit,
-                                                        lang="")
-                    except:
-                        searchresults = []
-
-                    for tweet in searchresults:
-                        username = tweet.user.GetScreenName()
-                        if userpictures:
-                            picture = tweet.user.GetProfileImageUrl()
-                        tweet.text = "%s: %s" % (username,
-                                                 tweet.text)
-                        tweet.picture = picture
-
-                    tweets += searchresults
-
-        # filter out message with curse words
         if filtertext:
-            for tweet in tweets:
+            filtertext = filtertext.lower()
+            filterlist = filtertext.split('\n')
+
+        # add picture and filter out tweets based on the filterlist
+        for tweet in tweets:
+            username = tweet.user.GetScreenName()
+            picture = tweet.user.GetProfileImageUrl()
+            tweet.text = "%s: %s" % (username,
+                                     tweet.text)
+            if userpictures:
+                tweet.picture = picture
+
+            if filtertext:
                 text = tweet.text.lower()
                 if not [1 for x in filterlist if x in text]:
                     results.append(tweet)
-            tweets = results
+            else:
+                results.append(tweet)
+        tweets = results
 
         # sort the tweets
         tweets.sort(key=lambda tweet: tweet.GetCreatedAtInSeconds())
